lumaagents

package module
v0.1.2 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 8, 2026 License: Apache-2.0 Imports: 14 Imported by: 0

README

Luma Go API Library

Go Reference

The Luma Go library provides convenient access to the Luma REST API from applications written in Go.

Installation

import (
	"github.com/lumalabs/luma-agents-go" // imported as lumaagents
)

Or to pin the version:

go get -u 'github.com/lumalabs/luma-agents-go@v0.1.2'

Requirements

This library requires Go 1.22+.

Usage

The full API of this library can be found in api.md.

package main

import (
	"context"
	"fmt"

	"github.com/lumalabs/luma-agents-go"
	"github.com/lumalabs/luma-agents-go/option"
)

func main() {
	client := lumaagents.NewClient(
		option.WithAuthToken("My Auth Token"), // defaults to os.LookupEnv("LUMA_AGENTS_API_KEY")
	)
	generation, err := client.Generations.New(context.TODO(), lumaagents.GenerationNewParams{
		Prompt:      lumaagents.F("A glass of iced coffee on a marble countertop, morning light streaming through a window"),
		AspectRatio: lumaagents.F(lumaagents.GenerationNewParamsAspectRatio16_9),
		Model:       lumaagents.F(lumaagents.ModelUni1),
	})
	if err != nil {
		panic(err.Error())
	}
	fmt.Printf("%+v\n", generation.ID)
}

Request fields

All request parameters are wrapped in a generic Field type, which we use to distinguish zero values from null or omitted fields.

This prevents accidentally sending a zero value if you forget a required parameter, and enables explicitly sending null, false, '', or 0 on optional parameters. Any field not specified is not sent.

To construct fields with values, use the helpers String(), Int(), Float(), or most commonly, the generic F[T](). To send a null, use Null[T](), and to send a nonconforming value, use Raw[T](any). For example:

params := FooParams{
	Name: lumaagents.F("hello"),

	// Explicitly send `"description": null`
	Description: lumaagents.Null[string](),

	Point: lumaagents.F(lumaagents.Point{
		X: lumaagents.Int(0),
		Y: lumaagents.Int(1),

		// In cases where the API specifies a given type,
		// but you want to send something else, use `Raw`:
		Z: lumaagents.Raw[int64](0.01), // sends a float
	}),
}
Response objects

All fields in response structs are value types (not pointers or wrappers).

If a given field is null, not present, or invalid, the corresponding field will simply be its zero value.

All response structs also include a special JSON field, containing more detailed information about each property, which you can use like so:

if res.Name == "" {
	// true if `"name"` is either not present or explicitly null
	res.JSON.Name.IsNull()

	// true if the `"name"` key was not present in the response JSON at all
	res.JSON.Name.IsMissing()

	// When the API returns data that cannot be coerced to the expected type:
	if res.JSON.Name.IsInvalid() {
		raw := res.JSON.Name.Raw()

		legacyName := struct{
			First string `json:"first"`
			Last  string `json:"last"`
		}{}
		json.Unmarshal([]byte(raw), &legacyName)
		name = legacyName.First + " " + legacyName.Last
	}
}

These .JSON structs also include an Extras map containing any properties in the json response that were not specified in the struct. This can be useful for API features not yet present in the SDK.

body := res.JSON.ExtraFields["my_unexpected_field"].Raw()
RequestOptions

This library uses the functional options pattern. Functions defined in the option package return a RequestOption, which is a closure that mutates a RequestConfig. These options can be supplied to the client or at individual requests. For example:

client := lumaagents.NewClient(
	// Adds a header to every request made by the client
	option.WithHeader("X-Some-Header", "custom_header_info"),
)

client.Generations.New(context.TODO(), ...,
	// Override the header
	option.WithHeader("X-Some-Header", "some_other_custom_header_info"),
	// Add an undocumented field to the request body, using sjson syntax
	option.WithJSONSet("some.json.path", map[string]string{"my": "object"}),
)

See the full list of request options.

Pagination

This library provides some conveniences for working with paginated list endpoints.

You can use .ListAutoPaging() methods to iterate through items across all pages:

Or you can use simple .List() methods to fetch a single page and receive a standard response object with additional helper methods like .GetNextPage(), e.g.:

Errors

When the API returns a non-success status code, we return an error with type *lumaagents.Error. This contains the StatusCode, *http.Request, and *http.Response values of the request, as well as the JSON of the error body (much like other response objects in the SDK).

To handle errors, we recommend that you use the errors.As pattern:

_, err := client.Generations.New(context.TODO(), lumaagents.GenerationNewParams{
	Prompt:      lumaagents.F("A glass of iced coffee on a marble countertop, morning light streaming through a window"),
	AspectRatio: lumaagents.F(lumaagents.GenerationNewParamsAspectRatio16_9),
	Model:       lumaagents.F(lumaagents.ModelUni1),
})
if err != nil {
	var apierr *lumaagents.Error
	if errors.As(err, &apierr) {
		println(string(apierr.DumpRequest(true)))  // Prints the serialized HTTP request
		println(string(apierr.DumpResponse(true))) // Prints the serialized HTTP response
	}
	panic(err.Error()) // GET "/generations": 400 Bad Request { ... }
}

When other errors occur, they are returned unwrapped; for example, if HTTP transport fails, you might receive *url.Error wrapping *net.OpError.

Timeouts

Requests do not time out by default; use context to configure a timeout for a request lifecycle.

Note that if a request is retried, the context timeout does not start over. To set a per-retry timeout, use option.WithRequestTimeout().

// This sets the timeout for the request, including all the retries.
ctx, cancel := context.WithTimeout(context.Background(), 5*time.Minute)
defer cancel()
client.Generations.New(
	ctx,
	lumaagents.GenerationNewParams{
		Prompt:      lumaagents.F("A glass of iced coffee on a marble countertop, morning light streaming through a window"),
		AspectRatio: lumaagents.F(lumaagents.GenerationNewParamsAspectRatio16_9),
		Model:       lumaagents.F(lumaagents.ModelUni1),
	},
	// This sets the per-retry timeout
	option.WithRequestTimeout(20*time.Second),
)
File uploads

Request parameters that correspond to file uploads in multipart requests are typed as param.Field[io.Reader]. The contents of the io.Reader will by default be sent as a multipart form part with the file name of "anonymous_file" and content-type of "application/octet-stream".

The file name and content-type can be customized by implementing Name() string or ContentType() string on the run-time type of io.Reader. Note that os.File implements Name() string, so a file returned by os.Open will be sent with the file name on disk.

We also provide a helper lumaagents.FileParam(reader io.Reader, filename string, contentType string) which can be used to wrap any io.Reader with the appropriate file name and content type.

Retries

Certain errors will be automatically retried 2 times by default, with a short exponential backoff. We retry by default all connection errors, 408 Request Timeout, 409 Conflict, 429 Rate Limit, and >=500 Internal errors.

You can use the WithMaxRetries option to configure or disable this:

// Configure the default for all requests:
client := lumaagents.NewClient(
	option.WithMaxRetries(0), // default is 2
)

// Override per-request:
client.Generations.New(
	context.TODO(),
	lumaagents.GenerationNewParams{
		Prompt:      lumaagents.F("A glass of iced coffee on a marble countertop, morning light streaming through a window"),
		AspectRatio: lumaagents.F(lumaagents.GenerationNewParamsAspectRatio16_9),
		Model:       lumaagents.F(lumaagents.ModelUni1),
	},
	option.WithMaxRetries(5),
)
Accessing raw response data (e.g. response headers)

You can access the raw HTTP response data by using the option.WithResponseInto() request option. This is useful when you need to examine response headers, status codes, or other details.

// Create a variable to store the HTTP response
var response *http.Response
generation, err := client.Generations.New(
	context.TODO(),
	lumaagents.GenerationNewParams{
		Prompt:      lumaagents.F("A glass of iced coffee on a marble countertop, morning light streaming through a window"),
		AspectRatio: lumaagents.F(lumaagents.GenerationNewParamsAspectRatio16_9),
		Model:       lumaagents.F(lumaagents.ModelUni1),
	},
	option.WithResponseInto(&response),
)
if err != nil {
	// handle error
}
fmt.Printf("%+v\n", generation)

fmt.Printf("Status Code: %d\n", response.StatusCode)
fmt.Printf("Headers: %+#v\n", response.Header)
Making custom/undocumented requests

This library is typed for convenient access to the documented API. If you need to access undocumented endpoints, params, or response properties, the library can still be used.

Undocumented endpoints

To make requests to undocumented endpoints, you can use client.Get, client.Post, and other HTTP verbs. RequestOptions on the client, such as retries, will be respected when making these requests.

var (
    // params can be an io.Reader, a []byte, an encoding/json serializable object,
    // or a "…Params" struct defined in this library.
    params map[string]interface{}

    // result can be an []byte, *http.Response, a encoding/json deserializable object,
    // or a model defined in this library.
    result *http.Response
)
err := client.Post(context.Background(), "/unspecified", params, &result)
if err != nil {
    …
}
Undocumented request params

To make requests using undocumented parameters, you may use either the option.WithQuerySet() or the option.WithJSONSet() methods.

params := FooNewParams{
    ID:   lumaagents.F("id_xxxx"),
    Data: lumaagents.F(FooNewParamsData{
        FirstName: lumaagents.F("John"),
    }),
}
client.Foo.New(context.Background(), params, option.WithJSONSet("data.last_name", "Doe"))
Undocumented response properties

To access undocumented response properties, you may either access the raw JSON of the response as a string with result.JSON.RawJSON(), or get the raw JSON of a particular field on the result with result.JSON.Foo.Raw().

Any fields that are not present on the response struct will be saved and can be accessed by result.JSON.ExtraFields() which returns the extra fields as a map[string]Field.

Middleware

We provide option.WithMiddleware which applies the given middleware to requests.

func Logger(req *http.Request, next option.MiddlewareNext) (res *http.Response, err error) {
	// Before the request
	start := time.Now()
	LogReq(req)

	// Forward the request to the next handler
	res, err = next(req)

	// Handle stuff after the request
	end := time.Now()
	LogRes(res, err, start - end)

    return res, err
}

client := lumaagents.NewClient(
	option.WithMiddleware(Logger),
)

When multiple middlewares are provided as variadic arguments, the middlewares are applied left to right. If option.WithMiddleware is given multiple times, for example first in the client then the method, the middleware in the client will run first and the middleware given in the method will run next.

You may also replace the default http.Client with option.WithHTTPClient(client). Only one http client is accepted (this overwrites any previous client) and receives requests after any middleware has been applied.

Semantic versioning

This package generally follows SemVer conventions, though certain backwards-incompatible changes may be released as minor versions:

  1. Changes to library internals which are technically public but not intended or documented for external use. (Please open a GitHub issue to let us know if you are relying on such internals.)
  2. Changes that we do not expect to impact the vast majority of users in practice.

We take backwards-compatibility seriously and work hard to ensure you can rely on a smooth upgrade experience.

We are keen for your feedback; please open an issue with questions, bugs, or suggestions.

Contributing

See the contributing documentation.

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Bool

func Bool(value bool) param.Field[bool]

Bool is a param field helper which helps specify bools.

func DefaultClientOptions

func DefaultClientOptions() []option.RequestOption

DefaultClientOptions read from the environment (LUMA_AGENTS_API_KEY, LUMA_BASE_URL). This should be used to initialize new clients.

func F

func F[T any](value T) param.Field[T]

F is a param field helper used to initialize a param.Field generic struct. This helps specify null, zero values, and overrides, as well as normal values. You can read more about this in our README.

func FileParam

func FileParam(reader io.Reader, filename string, contentType string) param.Field[io.Reader]

FileParam is a param field helper which helps files with a mime content-type.

func Float

func Float(value float64) param.Field[float64]

Float is a param field helper which helps specify floats.

func Int

func Int(value int64) param.Field[int64]

Int is a param field helper which helps specify integers. This is particularly helpful when specifying integer constants for fields.

func Null

func Null[T any]() param.Field[T]

Null is a param field helper which explicitly sends null to the API.

func Raw

func Raw[T any](value any) param.Field[T]

Raw is a param field helper for specifying values for fields when the type you are looking to send is different from the type that is specified in the SDK. For example, if the type of the field is an integer, but you want to send a float, you could do that by setting the corresponding field with Raw[int](0.5).

func String

func String(value string) param.Field[string]

String is a param field helper which helps specify strings.

Types

type Client

type Client struct {
	Options     []option.RequestOption
	Generations *GenerationService
}

Client creates a struct with services and top level methods that help with interacting with the luma API. You should not instantiate this client directly, and instead use the NewClient method instead.

func NewClient

func NewClient(opts ...option.RequestOption) (r *Client)

NewClient generates a new client with the default option read from the environment (LUMA_AGENTS_API_KEY, LUMA_BASE_URL). The option passed in as arguments are applied after these default arguments, and all option will be passed down to the services and requests that this client makes.

func (*Client) Delete

func (r *Client) Delete(ctx context.Context, path string, params interface{}, res interface{}, opts ...option.RequestOption) error

Delete makes a DELETE request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

func (*Client) Execute

func (r *Client) Execute(ctx context.Context, method string, path string, params interface{}, res interface{}, opts ...option.RequestOption) error

Execute makes a request with the given context, method, URL, request params, response, and request options. This is useful for hitting undocumented endpoints while retaining the base URL, auth, retries, and other options from the client.

If a byte slice or an io.Reader is supplied to params, it will be used as-is for the request body.

The params is by default serialized into the body using encoding/json. If your type implements a MarshalJSON function, it will be used instead to serialize the request. If a URLQuery method is implemented, the returned [url.Values] will be used as query strings to the url.

If your params struct uses param.Field, you must provide either [MarshalJSON], [URLQuery], and/or [MarshalForm] functions. It is undefined behavior to use a struct uses param.Field without specifying how it is serialized.

Any "…Params" object defined in this library can be used as the request argument. Note that 'path' arguments will not be forwarded into the url.

The response body will be deserialized into the res variable, depending on its type:

  • A pointer to a *http.Response is populated by the raw response.
  • A pointer to a byte array will be populated with the contents of the request body.
  • A pointer to any other type uses this library's default JSON decoding, which respects UnmarshalJSON if it is defined on the type.
  • A nil value will not read the response body.

For even greater flexibility, see option.WithResponseInto and option.WithResponseBodyInto.

func (*Client) Get

func (r *Client) Get(ctx context.Context, path string, params interface{}, res interface{}, opts ...option.RequestOption) error

Get makes a GET request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

func (*Client) Patch

func (r *Client) Patch(ctx context.Context, path string, params interface{}, res interface{}, opts ...option.RequestOption) error

Patch makes a PATCH request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

func (*Client) Post

func (r *Client) Post(ctx context.Context, path string, params interface{}, res interface{}, opts ...option.RequestOption) error

Post makes a POST request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

func (*Client) Put

func (r *Client) Put(ctx context.Context, path string, params interface{}, res interface{}, opts ...option.RequestOption) error

Put makes a PUT request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

type Error

type Error = apierror.Error

type Generation

type Generation struct {
	// Generation identifier
	ID string `json:"id" api:"required" format:"uuid"`
	// Creation timestamp
	CreatedAt string `json:"created_at" api:"required"`
	// Model used
	Model Model `json:"model" api:"required"`
	// Current state of the generation
	State GenerationState `json:"state" api:"required"`
	// The kind of generation to perform
	Type GenerationType `json:"type" api:"required"`
	// Machine-readable failure code for programmatic handling
	FailureCode GenerationFailureCode `json:"failure_code" api:"nullable"`
	// Human-readable failure description
	FailureReason string `json:"failure_reason" api:"nullable"`
	// Generated outputs (populated on completion)
	Output []GenerationOutput `json:"output"`
	JSON   generationJSON     `json:"-"`
}

Generation status and output

func (*Generation) UnmarshalJSON

func (r *Generation) UnmarshalJSON(data []byte) (err error)

type GenerationFailureCode

type GenerationFailureCode string

Machine-readable failure code for programmatic handling

const (
	GenerationFailureCodeContentModerated  GenerationFailureCode = "content_moderated"
	GenerationFailureCodeGenerationFailed  GenerationFailureCode = "generation_failed"
	GenerationFailureCodeBudgetExhausted   GenerationFailureCode = "budget_exhausted"
	GenerationFailureCodeOutputNotFound    GenerationFailureCode = "output_not_found"
	GenerationFailureCodeImageTooLarge     GenerationFailureCode = "image_too_large"
	GenerationFailureCodeUnsupportedFormat GenerationFailureCode = "unsupported_format"
	GenerationFailureCodeCorruptInput      GenerationFailureCode = "corrupt_input"
	GenerationFailureCodeInvalidRequest    GenerationFailureCode = "invalid_request"
	GenerationFailureCodeRateLimited       GenerationFailureCode = "rate_limited"
)

func (GenerationFailureCode) IsKnown

func (r GenerationFailureCode) IsKnown() bool

type GenerationNewParams

type GenerationNewParams struct {
	// Text prompt
	Prompt param.Field[string] `json:"prompt" api:"required"`
	// Output aspect ratio
	AspectRatio param.Field[GenerationNewParamsAspectRatio] `json:"aspect_ratio"`
	// Reference images for style/content guidance. Up to 9 for type 'image', up to 8
	// for type 'image_edit'.
	ImageRef param.Field[[]GenerationNewParamsImageRef] `json:"image_ref"`
	// Model identifier. `uni-1` is the default tier; `uni-1-max` produces
	// higher-quality output than `uni-1` at a higher per-image price. Both models are
	// available to all accounts — see Pricing for per-image rates.
	Model param.Field[Model] `json:"model"`
	// Output image format
	OutputFormat param.Field[GenerationNewParamsOutputFormat] `json:"output_format"`
	// Reference image for guided generation. Provide either url or inline base64 data
	// (not both).
	Source param.Field[GenerationNewParamsSource] `json:"source"`
	// Style preset (auto, manga)
	Style param.Field[GenerationNewParamsStyle] `json:"style"`
	// The kind of generation to perform
	Type param.Field[GenerationNewParamsType] `json:"type"`
	// Your end-user's stable opaque identifier (no PII). Forwarded to upstream model
	// providers as their per-user tagging field so trust & safety violations can be
	// attributed to a specific end-user rather than the whole API account. Also used
	// for per-end-user usage breakdowns in /v1/usage. Strongly recommended for partner
	// integrations.
	UserID param.Field[string] `json:"user_id"`
	// Enable web search grounding — the agent can search the web and download
	// reference images before generating.
	WebSearch param.Field[bool] `json:"web_search"`
}

func (GenerationNewParams) MarshalJSON

func (r GenerationNewParams) MarshalJSON() (data []byte, err error)

type GenerationNewParamsAspectRatio

type GenerationNewParamsAspectRatio string

Output aspect ratio

const (
	GenerationNewParamsAspectRatio3_1  GenerationNewParamsAspectRatio = "3:1"
	GenerationNewParamsAspectRatio2_1  GenerationNewParamsAspectRatio = "2:1"
	GenerationNewParamsAspectRatio16_9 GenerationNewParamsAspectRatio = "16:9"
	GenerationNewParamsAspectRatio3_2  GenerationNewParamsAspectRatio = "3:2"
	GenerationNewParamsAspectRatio1_1  GenerationNewParamsAspectRatio = "1:1"
	GenerationNewParamsAspectRatio2_3  GenerationNewParamsAspectRatio = "2:3"
	GenerationNewParamsAspectRatio9_16 GenerationNewParamsAspectRatio = "9:16"
	GenerationNewParamsAspectRatio1_2  GenerationNewParamsAspectRatio = "1:2"
	GenerationNewParamsAspectRatio1_3  GenerationNewParamsAspectRatio = "1:3"
)

func (GenerationNewParamsAspectRatio) IsKnown

type GenerationNewParamsImageRef

type GenerationNewParamsImageRef struct {
	// Base64-encoded image data
	Data param.Field[string] `json:"data"`
	// MIME type (e.g. image/jpeg). Required with data.
	MediaType param.Field[string] `json:"media_type"`
	// Publicly accessible image URL
	URL param.Field[string] `json:"url"`
}

Reference image for guided generation. Provide either url or inline base64 data (not both).

func (GenerationNewParamsImageRef) MarshalJSON

func (r GenerationNewParamsImageRef) MarshalJSON() (data []byte, err error)

type GenerationNewParamsOutputFormat

type GenerationNewParamsOutputFormat string

Output image format

const (
	GenerationNewParamsOutputFormatPng  GenerationNewParamsOutputFormat = "png"
	GenerationNewParamsOutputFormatJpeg GenerationNewParamsOutputFormat = "jpeg"
)

func (GenerationNewParamsOutputFormat) IsKnown

type GenerationNewParamsSource

type GenerationNewParamsSource struct {
	// Base64-encoded image data
	Data param.Field[string] `json:"data"`
	// MIME type (e.g. image/jpeg). Required with data.
	MediaType param.Field[string] `json:"media_type"`
	// Publicly accessible image URL
	URL param.Field[string] `json:"url"`
}

Reference image for guided generation. Provide either url or inline base64 data (not both).

func (GenerationNewParamsSource) MarshalJSON

func (r GenerationNewParamsSource) MarshalJSON() (data []byte, err error)

type GenerationNewParamsStyle

type GenerationNewParamsStyle string

Style preset (auto, manga)

const (
	GenerationNewParamsStyleAuto  GenerationNewParamsStyle = "auto"
	GenerationNewParamsStyleManga GenerationNewParamsStyle = "manga"
)

func (GenerationNewParamsStyle) IsKnown

func (r GenerationNewParamsStyle) IsKnown() bool

type GenerationNewParamsType

type GenerationNewParamsType string

The kind of generation to perform

const (
	GenerationNewParamsTypeImage     GenerationNewParamsType = "image"
	GenerationNewParamsTypeImageEdit GenerationNewParamsType = "image_edit"
)

func (GenerationNewParamsType) IsKnown

func (r GenerationNewParamsType) IsKnown() bool

type GenerationOutput

type GenerationOutput struct {
	// Media type (e.g. image)
	Type string `json:"type" api:"required"`
	// Presigned URL (1hr expiry)
	URL  string               `json:"url" api:"required" format:"uri"`
	JSON generationOutputJSON `json:"-"`
}

A single generated output

func (*GenerationOutput) UnmarshalJSON

func (r *GenerationOutput) UnmarshalJSON(data []byte) (err error)

type GenerationService

type GenerationService struct {
	Options []option.RequestOption
}

GenerationService contains methods and other services that help with interacting with the luma API.

Note, unlike clients, this service does not read variables from the environment automatically. You should not instantiate this service directly, and instead use the NewGenerationService method instead.

func NewGenerationService

func NewGenerationService(opts ...option.RequestOption) (r *GenerationService)

NewGenerationService generates a new service that applies the given options to each request. These options are applied after the parent client's options (if there is one), and before any request-specific options.

func (*GenerationService) Get

func (r *GenerationService) Get(ctx context.Context, generationID string, opts ...option.RequestOption) (res *Generation, err error)

Poll for generation status and output. On completion, the response includes presigned URLs to download the generated images.

func (*GenerationService) New

Submit an image generation or edit job. Returns immediately with an opaque job ID to poll via GET /generations/{id}.

type GenerationState

type GenerationState string

Current state of the generation

const (
	GenerationStateQueued     GenerationState = "queued"
	GenerationStateProcessing GenerationState = "processing"
	GenerationStateCompleted  GenerationState = "completed"
	GenerationStateFailed     GenerationState = "failed"
)

func (GenerationState) IsKnown

func (r GenerationState) IsKnown() bool

type GenerationType

type GenerationType string

The kind of generation to perform

const (
	GenerationTypeImage     GenerationType = "image"
	GenerationTypeImageEdit GenerationType = "image_edit"
)

func (GenerationType) IsKnown

func (r GenerationType) IsKnown() bool

type Model

type Model string

Model identifier. `uni-1` is the default tier; `uni-1-max` produces higher-quality output than `uni-1` at a higher per-image price. Both models are available to all accounts — see Pricing for per-image rates.

const (
	ModelUni1    Model = "uni-1"
	ModelUni1Max Model = "uni-1-max"
)

func (Model) IsKnown

func (r Model) IsKnown() bool

Directories

Path Synopsis

Jump to